#!/usr/bin/env python3
from cprint import cprint
import click

from pyp5js import commands

@click.group()
def command_line_entrypoint():
    """
    Function to define the entry point for the command line
    """
    pass


@command_line_entrypoint.command('new')
@click.argument('sketch_name')
@click.option('--sketch-dir', '-d', default=None)
def configure_new_sketch(sketch_name, sketch_dir):
    """
    Create dir and configure boilerplate

    Params:
    - sketch_name: name of the sketch (will create a {sketch_name}.py)

    Opitionals
    - sketch_dir: directory to save the sketch (defaults to {sketch_name})
    """
    sketch_py = commands.new_sketch(sketch_name, sketch_dir)

    cprint.ok(f"Your sketch was created!")
    cprint.ok(f"Please, open and edit the file {sketch_py} to draw. When you're ready to see your results, just run:")
    cmd = f"\t pyp5js transcrypt {sketch_name}"
    if sketch_dir:
        cmd += f" --sketch-dir {sketch_dir}"
    cprint.ok(cmd)


@command_line_entrypoint.command("transcrypt")
@click.argument("sketch_name")
@click.option('--sketch-dir', '-d', default=None)
def transcrypt_sketch(sketch_name, sketch_dir):
    """
    Command to generate the P5.js code for a python sketch

    Params:
    - sketch_name: name of the sketch

    Opitionals
    - sketch_dir: sketch's directory (defaults to {sketch_name})
    """
    index_file = commands.transcrypt_sketch(sketch_name, sketch_dir)
    cprint.ok(f"Your sketch is ready and available at {index_file}")


@command_line_entrypoint.command("monitor")
@click.argument("sketch_name")
@click.option('--sketch-dir', '-d', default=None)
def monitor_sketch(sketch_name, sketch_dir):
    """
    Command to generate keep watching a sketch's dir and, after any change,
    it'll automatically generate the JS files as in pyp5js transcrypt command

    Params:
    - sketch_name: name of the sketch

    Opitionals
    - sketch_dir: sketch's directory (defaults to ./{sketch_name})
    """
    commands.monitor_sketch(sketch_name, sketch_dir)

if __name__ == "__main__":
    command_line_entrypoint()



import os
from unipath import Path
from cprint import cprint
from jinja2 import Environment, FileSystemLoader


class Pyp5jsSketchFiles():

    def __init__(self, sketch_dir, sketch_name, check_sketch_dir=True):
        self._sketch_dir = sketch_dir or ''
        self.sketch_name = sketch_name
        self.check_sketch_dir = check_sketch_dir

    def can_create_sketch(self):
        if self.sketch_dir.exists():
            cprint.warn(f"Cannot configure a new sketch.")
            cprint.err(f"The directory {self.sketch_dir} already exists.", interrupt=True)

    def check_sketch_exists(self):
        return bool(self.sketch_dir and self.sketch_py)

    @property
    def sketch_dir(self):
        sketch_dir = Path(self._sketch_dir)

        if not sketch_dir:
            return sketch_dir.child(f'{self.sketch_name}')

        if self.check_sketch_dir and not sketch_dir.exists():
            cprint.err(f"The directory {sketch_dir} does not exists.", interrupt=True)

        return sketch_dir

    @property
    def static_dir(self):
        return self.sketch_dir.child('static')

    @property
    def index_html(self):
        return self.sketch_dir.child('index.html')

    @property
    def p5js(self):
        return self.static_dir.child('p5.js')

    @property
    def sketch_py(self):
        py_file = self.sketch_dir.child(f'{self.sketch_name}.py')

        if self.check_sketch_dir and not py_file.exists():
            cwd_py_file = Path(os.getcwd()).child(f"{sketch_name}.py")
            if not cwd_py_file.exists():
                cprint.warn(f"Couldn't find the sketch.")
                cprint.err(f"Neither the file {py_file} or {cwd_py_file} exist.", interrupt=True)

            py_file = cwd_py_file
            self._sketch_dir = py_file.parent

        return py_file

    @property
    def target_dir(self):
        return self.sketch_dir.child("target")


class Pyp5jsLibFiles():

    def __init__(self):
        self.install = Path(__file__).parent

    @property
    def templates_dir(self):
        return self.install.child('templates')

    @property
    def base_sketch(self):
        return self.templates_dir.child('base_sketch.py')

    @property
    def p5js(self):
        return self.install.child('static', 'p5.js')

    def render_new_index(self, context):
        templates = Environment(loader=FileSystemLoader(self.templates_dir))
        index_template = templates.get_template('index.html')
        return index_template.render(context)



import os
import shutil
import time
from cprint import cprint
from datetime import date
from unipath import Path
from watchdog.observers import Observer

from pyp5js.compiler import compile_sketch_js, TranscryptSketchEvent
from pyp5js.fs import Pyp5jsSketchFiles, Pyp5jsLibFiles

TARGET_DIRNAME = "target"


def new_sketch(sketch_name, sketch_dir):
    """
    Creates a new sketch, on a folder/directory
    with the required assets and a index.html file,
    all based on a template

    :param sketch_name: name for new sketch
    :type sketch_name: string
    :param sketch_dir: directory name
    :type sketch_dir: string
    :return: file names
    :rtype: list of strings
    """

    sketch_files = Pyp5jsSketchFiles(sketch_dir, sketch_name, check_sketch_dir=False)
    sketch_files.can_create_sketch()

    pyp5js_files = Pyp5jsLibFiles()
    templates_files = [
        (pyp5js_files.base_sketch, sketch_files.sketch_py),
        (pyp5js_files.p5js, sketch_files.p5js)
    ]

    context = {
        "p5_js_url": "static/p5.js",
        "sketch_js_url": f"{TARGET_DIRNAME}/{sketch_name}.js",
        "sketch_name": sketch_name,
    }
    index_contet = pyp5js_files.render_new_index(context)

    os.makedirs(sketch_files.sketch_dir)
    os.mkdir(sketch_files.static_dir)
    for src, dest in templates_files:
        shutil.copyfile(src, dest)

    with open(sketch_files.index_html, "w") as fd:
        fd.write(index_contet)

    return sketch_files.sketch_py


def transcrypt_sketch(sketch_name, sketch_dir):
    sketch_files = Pyp5jsSketchFiles(sketch_dir, sketch_name)
    sketch_files.check_sketch_exists()

    compile_sketch_js(sketch_files)
    return sketch_files.index_html


def monitor_sketch(sketch_name, sketch_dir):
    sketch_files = Pyp5jsSketchFiles(sketch_dir, sketch_name)
    sketch_files.check_sketch_exists()

    cprint(f"Monitoring for changes in {sketch_files.sketch_dir.absolute()}...")

    event_handler = TranscryptSketchEvent(sketch_files=sketch_files)
    observer = Observer()

    observer.schedule(event_handler, sketch_files.sketch_dir)
    observer.start()
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        cprint.info("Exiting monitor...")
        observer.stop()
    observer.join()



import yaml
from jinja2 import Environment, FileSystemLoader, select_autoescape
from unipath import Path
from cprint import cprint

PYP5_DIR = Path(__file__).parent
TEMPLATES_DIR = PYP5_DIR.child('templates')
ASSETS_DIR = PYP5_DIR.child('assets')

if __name__ == '__main__':
    with open(ASSETS_DIR.child('p5_reference.yml')) as fd:
        data = yaml.load(fd.read())
        methods_names = data['methods']
        event_function_names = data['events']
        variables_names = data['variables']

    templates = Environment(loader=FileSystemLoader(TEMPLATES_DIR))
    pyp5_template = templates.get_template('pytop5js.py.template')
    context = {
        'function_names': methods_names,
        'variables_names': variables_names,
        'event_function_names': event_function_names,
    }
    pyp5_content = pyp5_template.render(context)

    pyp5_path = PYP5_DIR.child('pytop5js.py')
    with open(pyp5_path, 'w') as fd:
        fd.write(pyp5_content)

    cprint.ok(f"File {pyp5_path} was updated with success.")



# Copyright 2018-2019 Bernardo Fontes <https://github.com/berinhard/pyp5js/>

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.

#    You should have received a copy of the GNU Lesser General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, find_packages

with open("requirements.txt") as fd:
    requirements = [l.strip() for l in fd.readlines()]

with open("README.md") as fd:
    long_description = fd.read()

setup(
    name="pyp5js",
    version="0.0.4",
    description='Simple tool to allow to transcrypt Python code that uses P5.js',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Bernardo Fontes',
    maintainer='Bernardo Fontes',
    maintainer_email='bernardoxhc@gmail.com',
    url="https://github.com/berinhard/pyp5js/",
    license='GNU Lesser General Public License version 3',
    packages=find_packages(),
    package_data={
        'pyp5js': [
            'assets/*',
            'static/*',
            'templates/*',
        ]
    },
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': ['pyp5js = pyp5js.cli:command_line_entrypoint']
    },
    python_requires='>=3.6',
    install_requires=requirements,
    keywords="p5js processing creative coding",
)



import subprocess
import shutil
import shlex
from cprint import cprint
from unipath import Path
from watchdog.events import PatternMatchingEventHandler


PYP5_DIR = Path(__file__).parent


def compile_sketch_js(sketch_files):
    command = ' '.join([str(c) for c in [
        'transcrypt', '-xp', PYP5_DIR, '-b', '-m', '-n', sketch_files.sketch_py
    ]])

    cprint.info(f"Converting Python to P5.js...\nRunning command:\n\t {command}")

    proc = subprocess.Popen(shlex.split(command))
    proc.wait()

    __target = sketch_files.sketch_dir.child('__target__')
    if not __target.exists():
        cprint.err(f"Error with transcrypt: the {__target} directory wasn't created.", interrupt=True)

    if sketch_files.target_dir.exists():
        shutil.rmtree(sketch_files.target_dir)
    shutil.move(__target, sketch_files.target_dir)


class TranscryptSketchEvent(PatternMatchingEventHandler):
    patterns = ["*.py"]

    def __init__(self, *args, **kwargs):
        self.sketch_files = kwargs.pop('sketch_files')
        self._last_event = None
        super().__init__(*args, **kwargs)

    def on_modified(self, event):
        event_id = id(event)
        if event_id == self._last_event:
            return

        cprint.info(f"New change in {event.src_path}")
        compile_sketch_js(self.sketch_files)
        index_file = self.sketch_files.index_html
        cprint.ok(f"Your sketch is ready and available at {index_file}")
